Player = class("Player", Entity)

function Player:initialize(name)
    Entity.initialize(self, name)

    self.maxHealth = 15
    self.health = 15
	
	self.img = love.graphics.newImage('img/player.png')
    self.portrait = love.graphics.newImage('img/portrait/male2.png')
	
	self.imgScale = 4
	self.width = self.img:getWidth()*self.imgScale
	self.height = self.img:getHeight()*self.imgScale

    self.strength = 5
    self.wisdom = 5
    self.agility = 5

    self.gold = 25
	
	self.backpack = {}
	
	self.location = nil
    self.lastTween = nil
	self.x = 0
	self.y = 0
	
	self.img = love.graphics.newImage('img/player.png')
end

-- Accepts a type of item (gold, weapon, etc) and gives/takes it to/from the player.
function Player:modifyInv(itemType, state, var1, var2, var3)
    local giveOrTake = 1

    if state == "give" then
        giveOrTake = 1
    elseif state == "take" then
        giveOrTake = -1
    end

    if itemType == "gold" then
        self.gold = self.gold + var1 * giveOrTake
    end

    -- can be refactored when string types are gone
    -- just insert an object(class) into backpack

	if state == "give" then
		if itemType == "weapon" then
            -- arg[1] should be a weapon
			table.insert(self.backpack, var1)

		-- This needs to be replaced with different types of items later on, such as armor and potions.
		elseif itemType == "item" then
			table.insert(self.backpack, {iType = "item", name = var1})
		end
	elseif state == "take" then
		if itemType == "weapon" then
			Player:removeFromBackpack("weapon", var1)
		-- This needs to be replaced with different types of items later on, such as armor and potions.
		elseif itemType == "item" then
			Player:removeFromBackpack("item", var1)
		end
	end
end

-- Checks for an item of a given type and name in the backpack, and returns true if it is found
-- If state is "remove" then it will delete the item as well
function Player:checkForItem(iType, name, state)
	for k, item in ipairs(self.backpack) do
		if item.iType == iType and item.name == name then
			if state and state == "remove" then
				table.remove(self.backpack, k)
			end
			return true
		end
	end
end

function Player:toSerialize()
    local table = {
        name = self.name,
        health = self.health,
        maxHealth = self.maxHealth,
        weapon = self.weapon,
        gold = self.gold,

        location = searchTable(game.map.list, self.location),
    }

    assert(table.location ~= nil)

    return table
end

function Player:fromSerialize(data)
    self.name = data.name
    self.health = data.health
    self.maxHealth = data.maxHealth
    self.weapon = data.weapon
    self.weapon.critRange = data.weapon.critRange
    self.gold = data.gold
    self.location = data.location

    assert(self.weapon ~= nil)
    assert(self.weapon.critRange ~= nil)
    assert(self.weapon.critRange[1] ~= nil)

    -- make sure map loaded properly
    assert(game.map ~= nil)
    assert(game.map.list ~= nil)
    assert(game.map.list[1] ~= nil)
    assert(self.location ~= nil)

    self.location = game.map.list[self.location]
    self.x = self.location.x - self.width/2
    self.y = self.location.y - self.height/2
end

-- moves the player to a different location on the map
function Player:moveTo(node, trigger)
	if self.lastTween ~= nil then
		tween.stop(self.lastTween)
	end

    -- whether or not to trigger that you entered the location
    if trigger == nil then trigger = true end

	self.location = node
	
	local newX = node.x - self.width/2
	local newY = node.y - self.height/2
	
	self.lastTween = tween.start(0.35, game.player, {x = newX, y = newY}, 'inOutQuad', function() 
		self.lastTween = nil

        if trigger then
            game:enteredLocation(self.location)
        end
	end)

    if self.location.type == "town" then
        self.lastTown = self.location
    end
end

-- takes care of what happens after you die
function Player:revive()
    -- lose half your gold (rounded donw)
    goldLost = math.floor(self.gold/2)

    self.gold = self.gold - goldLost

    -- revive with 10% of your max health (rounded up)
    self.health = math.ceil(self.maxHealth/10)

    -- revive at last town
    self:moveTo(self.lastTown, false)
end

function Player:draw()
	love.graphics.setColor(255, 255, 255)
	love.graphics.draw(self.img, self.x, self.y, 0, self.imgScale, self.imgScale)
end
 
