Input = class("Input")

function Input:initialize(x, y, w, h, align)
    self.text = ""
    self.font = font[32]
    self.x = x
    self.y = y
    self.width = w or self.font:getWidth(text)
    self.height = h or self.font:getHeight(text)
    self.align = align or "left"

    self.selected = false

    self.hovering = {33, 33, 33, 127}
    self.active = {0, 0, 0, 127}
    self.bg = {0, 0, 0, 64}
    self.fg = {0, 0, 0, 255}

    self.caretTimer = 0
end

function Input:textinput(text)
    if self.selected then
        self.text = self.text .. text
    end
end

function Input:keypressed(key, isrepeat)
    if key == "backspace" then
        self.text = string.sub(self.text, 1, -2)
    elseif key == "return" then
        self.selected = false
    end
end

function Input:draw()
    self.caretTimer = self.caretTimer + love.timer.getDelta()
    if self.caretTimer >= 1 then self.caretTimer = 0 end

    local r, g, b, a = love.graphics.getColor()
    local oldColor = {r, g, b, a}

    if self:hover() and not self.selected then
        love.graphics.setColor(self.hovering)
    elseif self.selected then
        love.graphics.setColor(self.active)
    else
        love.graphics.setColor(self.bg)
    end

    love.graphics.rectangle("fill", self.x, self.y, self.width, self.height)

    love.graphics.setColor(self.fg)

    local x = self.x + 15 --+ self.width/2 - self.font:getWidth(self.text)/2
    local y = self.y + self.height/2 - self.font:getHeight(self.text)/2
    love.graphics.setFont(self.font)

    -- placeholder text
    if not self.selected and self.text == "" then
        love.graphics.setColor({255, 255, 255, 127})
        x = x + (self.width/2 - self.font:getWidth("Enter your name")/2) - 15
        love.graphics.print("Enter your name", x, y)
    else

        if self.align == "left" then
            love.graphics.print(self.text, x, y)
        elseif self.align == "center" then
            x = x + (self.width/2 - self.font:getWidth(self.text)/2) - 15
            love.graphics.print(self.text, x, y)
        end
    end

    -- blinking caret
    if self.selected then
        love.graphics.setLineWidth(1)

        local alpha = 0

        if self.caretTimer > 0.5 then
            alpha = 255
        else
            alpha = 0
        end

        love.graphics.setColor(0, 0, 0, alpha)
        love.graphics.line(
            x + self.font:getWidth(self.text)+2, y+5,
            x + self.font:getWidth(self.text)+2, y+self.font:getHeight(self.text)-5
        )
    end

    love.graphics.setColor(oldColor)
end

function Input:setFont(font)
    self.font = font
    self.width =  self.font:getWidth(self.text)
    self.height = self.font:getHeight(self.text)
    return self
end

function Input:setBG(r, g, b, a)
    self.bg = {r, g, b, a or 255}
    return self
end

function Input:setFG(r, g, b, a)
    self.fg = {r, g, b, a or 255}
    return self
end

--[[function Input:align(mode)
    -- this will center align the button around the original coordinates
    if mode == 'both' then
        self.x = self.x - self.width/2
        self.y = self.y - self.height/2
    end

    -- this will only align the button around the original x coordinate
    if mode == 'x' then
        self.x = self.x - self.width/2
    end

    -- this will only align the button around the original y coordinate
    if mode == 'y' then
        self.y = self.y - self.height/2
    end
end]]

function Input:centerAround(x, y)
    self.x = x - self.width/2
    self.y = y - self.height/2
    return self
end

function Input:hover()
    local xBound = love.mouse.getX() > self.x and love.mouse.getX() < self.x+self.width
    local yBound = love.mouse.getY() > self.y and love.mouse.getY() < self.y+self.height

    return xBound and yBound
end

Input.clicked = Input.hover