overworldSelector = class("overworldSelector")

-- List should contain objects with x, y, width, and height values. The selector can iterate through all of these objects
-- Color is a table {r, g, b, [a]}
-- Current is the currently selected item in List (defaults to 1)
-- Segmented is a boolean (whether the selector should use quads or not). Use false for a single image selector
-- Blink is a boolean, if true then the selector will blink on and off at a fixed rate
function overworldSelector:initialize(list, color, imgRef)
	self.list = list
	self.color = color or {255, 255, 255, 255}
    self.moveIn = true
	
	self.lastTween = nil
	
	self.x = love.graphics.getWidth()/2
	self.y = love.graphics.getWidth()/2
	self.velocityX = 0
	self.velocityY = 0
	self.speed = 40
	
	self.attached = false
	self.attachedNode = nil
	self.attachThreshold = 35

	self.justAttachedTimer = 0
	
	self.handleDist = 20
	
	self.img = imgRef and love.graphics.newImage(imgRef) or love.graphics.newImage('img/ui/gameSelector.png')
	self.imgWidth = self.img:getWidth()
	self.imgHeight = self.img:getHeight()
	
	self.quad1 = love.graphics.newQuad(0, 0, self.imgWidth/2, self.imgHeight/2, self.imgWidth, self.imgHeight)
	self.quad2 = love.graphics.newQuad(self.imgWidth/2, 0, self.imgWidth/2, self.imgHeight/2, self.imgWidth, self.imgHeight)
	self.quad3 = love.graphics.newQuad(0, self.imgHeight/2, self.imgWidth/2, self.imgHeight/2, self.imgWidth, self.imgHeight)
	self.quad4 = love.graphics.newQuad(self.imgWidth/2, self.imgHeight/2, self.imgWidth/2, self.imgHeight/2, self.imgWidth, self.imgHeight)
end

function overworldSelector:update(dt)
	if love.keyboard.isDown("up") then self.velocityY = self.velocityY - self.speed end
	if love.keyboard.isDown("down") then self.velocityY = self.velocityY + self.speed end
	if love.keyboard.isDown("left") then self.velocityX = self.velocityX - self.speed end
	if love.keyboard.isDown("right") then self.velocityX = self.velocityX + self.speed end

	self.velocityX = self.velocityX * 0.85
	self.velocityY = self.velocityY * 0.85

	
	local screenWidth = love.graphics.getWidth()
	local screenHeight = love.graphics.getHeight()
	if self.x + self.velocityX <= screenWidth/7 or self.x + self.velocityX >= screenWidth*6/7 or self.y + self.velocityY <= screenHeight/7 or self.y + self.velocityY >= screenHeight*6/7 then
		game.camera.x = game.camera.x + self.velocityX * dt
		game.camera.y = game.camera.y + self.velocityY * dt
	else
		self.x = self.x + self.velocityX * dt
		self.y = self.y + self.velocityY * dt
	end

	self.justAttachedTimer = self.justAttachedTimer - dt

	local closestNode, distance = game.map:closestNode(self.x+game.camera.x, self.y+game.camera.y, self.attachThreshold)

	local didNotJustAttach = self.justAttachedTimer <= 0
	local closeEnough = distance <= self.attachThreshold

	if didNotJustAttach and closeEnough and not self.attached then
		self.attachedNode = closestNode
		self.attached = true

		self.velocityX = 0
		self.velocityY = 0
	end

	if self.attached and not love.keyboard.isDown("up", "down", "left", "right") then
		self.x = self.attachedNode.x - game.camera.x
		self.y = self.attachedNode.y - game.camera.y
	end

	if self.attached then
		self.handleDist = self.attachedNode.radius
	end

	if self.attached and math.sqrt(self.velocityX^2 + self.velocityY^2) > 150 then
		self.attached = false
		self.attachedNode = nil

		self.justAttachedTimer = 0.25
	end

	if self.attached and distance > self.attachThreshold then
		self.attached = false
		self.attachedNode = nil

		self.justAttachedTimer = 0.25
	end
end

function overworldSelector:keypressed(key, isrepeat)
	if self.attached and (key == " " or key == "return" or key == "z") then

		if game.player.location ~= self.attachedNode and game.player.location:connectedTo(self.attachedNode) then
			game.player:moveTo(self.attachedNode)
		end
	end
end

function overworldSelector:draw()
	
	local oldColor = {love.graphics.getColor()}
	

	love.graphics.setColor(self.color)
	
	local centerDist = self.handleDist
	love.graphics.draw(self.img, self.quad1, self.x-centerDist, self.y-centerDist)
	love.graphics.draw(self.img, self.quad2, self.x+centerDist-self.imgWidth/2, self.y-centerDist)
	love.graphics.draw(self.img, self.quad3, self.x-centerDist, self.y+centerDist-self.imgHeight/2)
	love.graphics.draw(self.img, self.quad4, self.x+centerDist-self.imgWidth/2, self.y+centerDist-self.imgHeight/2)


	love.graphics.setColor(255, 0, 0)

	--love.graphics.line(game.player.x, game.player.y, game.player.x-game.camera.x, game.player.y-game.camera.y)
	
	love.graphics.setColor(oldColor)
end