local Text = class("Text")

-- Color should be a table: {r, g, b, a}
function Text:initialize(line, x, y, color, width, align, r, tag)
	if line then
		line = string.gsub(line, "$PlayerName", game.player.name)
		line = string.gsub(line, "$TownName", "Guilder")
		
		self.strng = line
	else
		self.strng = "no text"
	end
	
	self.x = x or 0
	self.y = y or 0
	self.color = color or {255, 0, 155, 255}
	self.width = width or 400
	self.align = align or "left"
	self.r = r or 0
	
	self.charDisp = 1
	self.timerMax = .03
	self.timerCurrent = .05
	
	self.tag = tag or ""
end

function Text:draw()
	--[[
	love.graphics.setColor(100, 20, 200)
	love.graphics.roundrectangle("fill", self.x, self.y, self.width, 200, 20)
	love.graphics.setColor(255, 20, 15)
	love.graphics.roundrectangle("line", self.x, self.y, self.width, 200, 20)
	]]
	
	love.graphics.setColor(self.color)
	--love.graphics.print(self.strng, self.x, self.y)
	
	if self.charDisp > 0 then
		local printString = string.sub(self.strng, 1, self.charDisp)
		love.graphics.printf(printString, self.x, self.y, self.width, self.align, self.r)
	end
end


texter = {}
local textList = {}

function texter:add(strng, x, y, color, width, align, r, tag)
	table.insert(textList, Text:new(strng, x, y, color, width, align, r, tag))
end

function texter:clear()
	textList = {}
end

function texter:removeTagged(tag)
	for i = #textList, 1, -1 do
		if textList[i].tag == tag then
			table.remove(textList, i)
		end
	end
end

function texter:update(dt)
	for k, textValue in pairs(textList) do
		local strng = textValue.strng
		local charDisp = textValue.charDisp
		local timerMax = textValue.timerMax
		local timerCurrent = textValue.timerCurrent
		
		timerCurrent = timerCurrent - dt
		textList[k].timerCurrent = timerCurrent
		if timerCurrent <= 0 then
			textList[k].timerCurrent = timerMax
			if charDisp < string.len(strng) then
				textList[k].charDisp = charDisp + 1
			end
		end
	end
end

function texter:draw()
	local r,g,b,a = love.graphics.getColor()
	for k, textData in pairs(textList) do
		textData:draw()
	end
	love.graphics.setColor(r,g,b,a)
end




-- Temp location
-- Found here: http://love2d.org/forums/viewtopic.php?f=4&t=77599

--[[
function love.graphics.roundrectangle(mode, x, y, w, h, rd, s)
   local r, g, b, a = love.graphics.getColor()
   local rd = rd or math.min(w, h)/4
   local s = s or 32
   local l = love.graphics.getLineWidth()
   
   local corner = 1
   local function mystencil()
      love.graphics.setColor(255, 255, 255, 255)
      if corner == 1 then
         love.graphics.rectangle("fill", x-l, y-l, rd+l, rd+l)
      elseif corner == 2 then
         love.graphics.rectangle("fill", x+w-rd+l, y-l, rd+l, rd+l)
      elseif corner == 3 then
         love.graphics.rectangle("fill", x-l, y+h-rd+l, rd+l, rd+l)
      elseif corner == 4 then
         love.graphics.rectangle("fill", x+w-rd+l, y+h-rd+l, rd+l, rd+l)
      elseif corner == 0 then
         love.graphics.rectangle("fill", x+rd, y-l, w-2*rd+l, h+2*l)
         love.graphics.rectangle("fill", x-l, y+rd, w+2*l, h-2*rd+l)
      end
   end
   
   love.graphics.setStencil(mystencil)
   love.graphics.setColor(r, g, b, a)
   love.graphics.circle(mode, x+rd, y+rd, rd, s)
   love.graphics.setStencil()
   corner = 2
   love.graphics.setStencil(mystencil)
   love.graphics.setColor(r, g, b, a)
   love.graphics.circle(mode, x+w-rd, y+rd, rd, s)
   love.graphics.setStencil()
   corner = 3
   love.graphics.setStencil(mystencil)
   love.graphics.setColor(r, g, b, a)
   love.graphics.circle(mode, x+rd, y+h-rd, rd, s)
   love.graphics.setStencil()
   corner = 4
   love.graphics.setStencil(mystencil)
   love.graphics.setColor(r, g, b, a)
   love.graphics.circle(mode, x+w-rd, y+h-rd, rd, s)
   love.graphics.setStencil()
   corner = 0
   love.graphics.setStencil(mystencil)
   love.graphics.setColor(r, g, b, a)
   love.graphics.rectangle(mode, x, y, w, h)
   love.graphics.setStencil()
end
]]


return texter


