
-- libraries
class = require 'libs.middleclass'
vector = require 'libs.vector'
state = require 'libs.state'
tween = require 'libs.tween'
fx = require 'libs.fx'

texter = require 'libs.text'

-- global libraries (the names within shouldn't change)
require 'libs.util'
require 'libs.generalmath'

-- entities (ui should move to libs eventually)
require 'entities.ui.button'
require 'entities.ui.bar'
require 'entities.ui.input'
require 'entities.ui.selector'
require 'entities.ui.overworldSelector'
require 'entities.ui.targetSelector'

require 'entities.nodes'
require 'entities.entity'
require 'entities.enemy'
require 'entities.player'
require 'entities.item'

require 'entities.combatTarget'

-- gamestates
require 'states.menu'
require 'states.start'
require 'states.game'
require 'states.load'
require 'states.save'
require 'states.pause'
require 'states.status'
require 'states.combat'
require 'states.combatReward'
require 'states.town'
require 'states.tavern'
require 'states.shop'
require 'states.event'
require 'states.death'

function love.load()
	love.window.setTitle(config.windowTitle)
    love.window.setIcon(love.image.newImageData(config.windowIcon))
	love.graphics.setDefaultFilter(config.filterModeMin, config.filterModeMax, config.anisotropy)
    love.graphics.setFont(font[14])

    math.randomseed(os.time()/10)

    -- Some objects need to be initialized earlier because it can be used before the
    -- game actually starts (start screen, saving/loading)
    game.player = Player:new()
    game.map = NodeGroup:new()

    state.registerEvents()
    state.switch(menu)
end

function love.update(dt)
    tween.update(dt)
end

function love.draw()

end

function love.keypressed(key, isrepeat)

    -- play a sound when you click a button
    local currentState = state.current()
    if currentState.selector ~= nil then
        if currentState.selector.currentElement:isInstanceOf(Button) then
            if key == " " or key == "return" then
                playSound(Button.click)
            end
        end
    end
end