combat = {}

combat.buttons = {}
combat.targets = {}
combat.enemies = {}
combat.entities = {}

CombatButton = class("CombatButton", Button)
function CombatButton:initialize(text, x, y, w, h)
    Button.initialize(self, text, x, y, w, h)

    self:setFont(font[32])
    self:align("y")
    self:setBG(22, 22, 22, 0)
    self:setFG(255, 255, 255)

    table.insert(combat.buttons, self)
end

function combat:init()
    -- modes:
    --  normal
    --  targetSelect

    self.mode = "normal"

    self.action = nil
    self.selectedTarget = nil

    --self.sound.theme = love.audio.newSource('sound/soliloquy.mp3', 'stream')
    --self.sound.themeVolume = 0.0

    --playSound(self.sound.theme, self.sound.themeVolume)

    self.banner = love.graphics.newImage('img/bg/battle.png')
    self.backgroundImage = love.graphics.newImage('img/bg/papyrus.jpg')

    -- baseline to place buttons on
    self.buttonLine = love.graphics.getHeight() - (love.graphics.getHeight() - self.banner:getHeight())/4

    -- baseline to put frame on
    self.frameLine = (love.graphics.getHeight() - self.banner:getHeight())/2

    self.margin = 75

    self.attackButton = CombatButton("Attack", 30, self.buttonLine)
    self.attackButton.activated = function()
        self.mode = "targetSelect"
        self.action = "attack"
    end

    -- calculate buttons width
    self.buttonsWidth = 0
    for k, button in pairs(self.buttons) do
        self.buttonsWidth = self.buttonsWidth + button.width
    end
    self.buttonsWidth = self.buttonsWidth + self.margin*(#self.buttons-1)

    -- center buttons around center of the screen
    for k, button in pairs(self.buttons) do
        button.x = button.x + love.graphics.getWidth()/2 - self.buttonsWidth/2
    end

    self.frameImage = love.graphics.newImage('img/ui/frame.png')
    self.frameImage:setWrap('repeat', 'repeat')
    self.frameQuad = love.graphics.newQuad(0, 0, love.graphics.getWidth(), self.frameImage:getHeight(), self.frameImage:getWidth(), self.frameImage:getHeight())

    self.actionSelector = Selector:new(self.buttons)
end

function combat:enter()
    love.graphics.setBackgroundColor(22, 22, 22)

    self.targets = {}
    self.enemies = {}
    self.entities = {}

    self.damageMessages = {}

    -- info relevant to combat rewards
    self.combatInfo = {}

    -- enemies
    local enemyAmount = math.random(3)
    self.combatInfo.enemies = enemyAmount

    for i=1, enemyAmount do
        table.insert(self.entities, Enemy:new("enemy name"))
    end

    -- targets
    local enemyPlacementLine = function(x)
        return love.graphics.getHeight()/2 - (love.graphics.getWidth()/2 + 34 - x/2) + 75
    end

    for k, entity in pairs(self.entities) do
        if entity ~= game.player then
            local x = love.graphics.getWidth()/2 + 100*(k-1) + 75
            local y = enemyPlacementLine(x)
            local enemy = CombatTarget:new(entity, x, y)
            table.insert(self.targets, enemy)
            table.insert(self.enemies, enemy)
        end
    end

    self.playerTarget = PlayerTarget:new(game.player, 75, love.graphics.getHeight()/2)
    self.playerTarget.y = self.playerTarget.y - self.playerTarget.height/2
    table.insert(self.targets, self.playerTarget)

    table.insert(self.entities, game.player)

    self.targetSelector = TargetSelector:new(self.enemies, 1, "horizontal", nil, nil, nil, 'img/ui/selectorArrow.png')
	
	if math.random() > .1 then
		state.push(event)
	end
end

function combat:enemiesAttack(dt, lastEnemy)
    lastEnemy = lastEnemy or 1

    self:checkDead()

    if lastEnemy > #self.enemies then
        return
    end

    enemy = self.enemies[lastEnemy]

    local target = self.playerTarget
    local oldX, oldY = enemy.x, enemy.y
    local newX, newY = target.x + target.width + 20, target.y - target.height

    self.lastTween = tween(0.35, enemy, {x=newX, y=newY}, "inOutQuad", function(self, enemy)

        local damageDealt, critical = enemy.entity:attack(target.entity)

        table.insert(self.damageMessages, {damageDealt, target.x + target.width/2, target.y - 20, 0, critical})

        target.color = {255, 0, 0}

        tween(0.2, target.color, {255, 255, 255}, nil, function(self, enemy)
            tween(0.3, enemy, {x=oldX, y=oldY}, "inOutQuad", function(self, enemy)
                self.lastTween = nil
                self:enemiesAttack(dt, lastEnemy+1)
            end, self, enemy)
        end, self, enemy)
    end, self, enemy)

    if lastEnemy == #self.enemies then
        return
    end
end

function combat:checkDead(dt)
    for k, target in pairs(self.targets) do
        
        -- the entity is dead, remove it
        if target.entity.health <= 0 then
            table.remove(self.entities, searchTable(self.entities, target.entity))
            table.remove(self.enemies, searchTable(self.enemies, target))
            table.remove(self.targets, searchTable(self.targets, target))

            if #self.enemies > 0 then
                self.targetSelector:selectNext()
            end
        end

        target:update(dt)
    end
end

function combat:update(dt)

    if game.player.health <= 0 then
        game.player.health = 0
        fx.transition(0.35, death)
        return
    end

    -- all enemies are dead, combat is done

    -- for some unknown reason, when #self.enemies is 0 it's actually 1.
    -- i have no idea why this is happening and i am just going to let it be
    if #self.enemies == 0 then
        fx.transition(0.35, combatReward)
        return
    end

    if self.mode == "normal" then
       self.actionSelector:update(dt)
    elseif self.mode == "targetSelect" and self.targetSelector ~= nil and #self.enemies > 0 then
        self.targetSelector:update(dt)
    end

    -- target has been selected already
    if self.selectedTarget ~= nil and #self.enemies > 0 then

        -- simulate a round of attacking
        if self.action == "attack" and self.lastTween == nil then
            -- animation? (WIP)
            local oldX, oldY = self.playerTarget.x, self.playerTarget.y
            local newX, newY = self.selectedTarget.x - 90, self.selectedTarget.y + self.selectedTarget.height/2
            local target = self.selectedTarget
            local ent = self.selectedTarget.entity

            self.playerTarget.img = self.playerTarget.attackImg

            self.lastTween = tween(0.5, self.playerTarget, {x=newX, y=newY}, "inOutQuad", function(self)
                combat.playerTarget.img = combat.playerTarget.swingImg

                local damageDealt, critical = game.player:attack(ent)

                table.insert(self.damageMessages, {damageDealt, target.x + target.width/2, target.y, 0, critical})

                target.color = {255, 0, 0}

                tween(0.25, target.color, {255, 255, 255}, nil, function(self)
                    combat.playerTarget.img = combat.playerTarget.idleImg
                    tween(0.4, combat.playerTarget, {x=oldX, y=oldY}, "inOutQuad", function()
                        self.lastTween = nil
                        self:enemiesAttack(dt)
                    end, self)
                end, self)
            end, self)

            self:checkDead()
            

            self.action = nil
            self.selectedTarget = nil
            self.mode = "normal"
        end
    end

    -- damage notifications
    for k, message in pairs(self.damageMessages) do
        local msg = message[1]
        local x, y = message[2], message[3]
        local t = message[4]

        t = t + dt

        y = y - 25 * dt

        message[2], message[3] = x, y
        message[4] = t

        if t > 1.5 then
            table.remove(self.damageMessages, searchTable(self.damageMessages, message))
        end
    end
end

function combat:keypressed(key, isrepeat)
    if #self.enemies == 0 then return end

    if self.mode == "normal" then
        self.actionSelector:keypressed(key, isrepeat)
    elseif self.mode == "targetSelect" and self.targetSelector ~= nil then
        self.targetSelector:keypressed(key, isrepeat)
    end
end


function combat:mousepressed(x, y, button)
	for k, target in ipairs(self.targets) do
		if k < #self.targets then
			target:mousepressed(button)
		end
    end
end

function combat:draw()
    love.graphics.setColor(255, 255, 255)
	
	
    -- background image
    local sx = love.graphics.getWidth()/self.backgroundImage:getWidth()
    local sy = love.graphics.getHeight()/self.backgroundImage:getHeight()
    love.graphics.draw(self.backgroundImage, 0, 0, nil, sx, sy)
	

    -- banner image
    love.graphics.setColor(255, 255, 255)
    local x = 0
    local y = (love.graphics.getHeight() - self.banner:getHeight())/2
    local sx = love.graphics.getWidth()/self.banner:getWidth()

    -- this is the distance from the top frame line to the bottom frame line
    -- this allows us to find how much we should scale the height
    local frameLineDistY = (love.graphics.getHeight() - 2*self.frameLine - self.frameImage:getHeight()/2)

    local sy = frameLineDistY / self.banner:getHeight()
    love.graphics.draw(self.banner, x, y, nil, sx, sy)
	

    -- image frames
    love.graphics.draw(self.frameImage, self.frameQuad, 0, self.frameLine-self.frameImage:getHeight()/2)
    love.graphics.draw(self.frameImage, self.frameQuad, 0, love.graphics.getHeight()-self.frameLine-self.frameImage:getHeight()/2)

    -- buttons
    self.attackButton:draw()

    if self.mode == "normal" then
       self.actionSelector:draw()
    elseif self.mode == "targetSelect" and self.targetSelector ~= nil then
        self.targetSelector:draw()
    end

    -- player portrait
    love.graphics.draw(game.player.portrait, 10, 7)

    -- player status (hp)
    local hpRatio = game.player.health / game.player.maxHealth
    love.graphics.setColor(216, 50, 47)
    love.graphics.rectangle("fill", 117, 87, 150*hpRatio, 20)
    love.graphics.setColor(255, 255, 255)
    love.graphics.setFont(font[16])
    local w = love.graphics.getFont():getWidth(game.player.health..' / '..game.player.maxHealth)
    love.graphics.print(game.player.health..' / '..game.player.maxHealth, 192-w/2, 85)

    -- targets
    for k, target in ipairs(self.targets) do
        target:draw()
    end

    -- damage notifications
    for k, message in pairs(self.damageMessages) do
        local msg = message[1]
        local x, y = message[2], message[3]
        local t = message[4]
        local critical = message[5]

        local completion = t/1.5

        if critical then
            love.graphics.setColor(255, 0, 0, 255 - completion*255)
        else
            love.graphics.setColor(255, 255, 255, 255 - completion*255)
        end

        love.graphics.setFont(font[48])
        love.graphics.print(msg, x, y)
    end

    love.graphics.setColor(255, 255, 255)

    -- select a target text
    if self.mode == "targetSelect" then
        love.graphics.print("Select a target", love.graphics.getWidth()/2 - love.graphics.getFont():getWidth("Select a target")/2, 40)
    end

    --[[
    love.graphics.print(self.mode, 5, 5)
    if self.action ~= nil then
        love.graphics.print(self.action, 5, 50)
    end
    if self.selectedTarget ~= nil then
        love.graphics.print(tostring(self.selectedTarget), 5, 95)
    end]]

    fx.draw()
end