combatReward = {}

combatReward.sound = {}

combatReward.uiElements = {}

function combatReward:init()
    self.backgroundImage = love.graphics.newImage('img/bg/sand_papyrus.jpg')

    self.continueButton = Button("Continue", love.graphics.getWidth()/2, 700)
    self.continueButton:align('x')
    self.continueButton:setFG(255, 255, 255)
    self.continueButton:setBG(0, 0, 0, 0)
    self.continueButton:setFont(font[36])
    self.continueButton.activated = function()
        fx.transition(0.35, game)
    end
    table.insert(self.uiElements, self.continueButton)

    self.sound.theme = love.audio.newSource('sound/coinsFalling.wav', 'static')

    self.goldImage = love.graphics.newImage('img/gold.png')
    self.goldQuad = love.graphics.newQuad(96, 32, 32, 32, self.goldImage:getDimensions())
	self.coinQuad = love.graphics.newQuad(0, 0, 32, 32, self.goldImage:getDimensions())
	
    self.selector = Selector:new(self.uiElements)
end

function combatReward:enter(prev, data)
    self.combatInfo = prev.combatInfo

    self.goldReward = 10 + 3*self.combatInfo.enemies

    self.goldReward = self.goldReward + math.random(1, 5)

    --game.player.gold = game.player.gold + self.goldReward
	game.player:modifyInv("gold", "give", self.goldReward)
	
    love.graphics.setBackgroundColor(22, 22, 22)
	
	self.activeCoins = {}
	self.coinAppearCounter = 0
	self.coinAppearCounterStep = 0

    self.sound.themeVolume = 0.3

    playSound(self.sound.theme, self.sound.themeVolume)

end

function combatReward:leave()
    self.sound.theme:stop()
end

function combatReward:update(dt)
    self.selector:update(dt)
	if self.activeCoins then
		if self.coinAppearCounter <= 0 and self.coinAppearCounterStep < self.goldReward then
			self.coinAppearCounter = .2
			self.coinAppearCounterStep = self.coinAppearCounterStep + 1
			
			table.insert(self.activeCoins, {x = 0, y = 0, fade = 255})
			
			tween.start(.25, self.activeCoins[#self.activeCoins], {y = 90, fade = 0}, 'inOutQuad', function() 
			table.remove(self.activeCoins[#self.activeCoins])
			end)
		end
		self.coinAppearCounter = self.coinAppearCounter - dt
	end

end

function combatReward:keypressed(key, isrepeat)
    self.selector:keypressed(key, isrepeat)
end

function combatReward:drawGold()
    love.graphics.push()

    local text = "+"..tostring(self.goldReward).." gold"
    love.graphics.setFont(font[32])

    local scaleFactor = 2
    local goldImageWidth = 32*scaleFactor
    local margin = 8
    local textWidth = love.graphics.getFont():getWidth(text)
    local sum = goldImageWidth + margin + textWidth

    love.graphics.translate(love.graphics.getWidth()/2 - sum/2, 0)

    love.graphics.print(text, sum - textWidth, 400)

    local x = 0
    local y = 400 - 16
    love.graphics.draw(self.goldImage, self.goldQuad, x, y, nil, 2, 2)
	
	for k, coin in pairs(self.activeCoins) do
		local r, g, b = love.graphics.getColor()
		love.graphics.setColor(r, g, b, coin.fade)
		love.graphics.draw(self.goldImage, self.coinQuad, coin.x, y - 100 + coin.y, nil, 2, 2)
	
		love.graphics.setColor(r, g, b, 255)
	end
	
    love.graphics.pop()
end

function combatReward:draw()
    love.graphics.setColor(255, 255, 255)

    -- background image for top and bottom bars
    local sx = love.graphics.getWidth()/self.backgroundImage:getWidth()
    local sy = love.graphics.getHeight()/self.backgroundImage:getHeight()
    love.graphics.draw(self.backgroundImage, 0, 0, nil, sx, sy)

    -- victory text
    local text = "VICTORY!"
    love.graphics.setFont(font[56])
    love.graphics.print(text, love.graphics.getWidth()/2 - love.graphics.getFont():getWidth(text)/2, 75)

    self:drawGold()

    self.continueButton:draw()

    self.selector:draw()

    fx.draw()
end