game = {}

game.mode = "normal"
game.buttons = {}
game.uiElements = {}

game.camera = {x = 0, y = 0}

game.townVisible = false
game.exitVisible = false

GameButton = class("MenuButton", Button)
function GameButton:initialize(text, x, y, w, h)
    Button.initialize(self, text, x, y, w, h)

    self:setFont(font[32])
    self:align("y")
    self:setBG(22, 22, 22, 0)
    self:setFG(255, 255, 255)

    table.insert(game.buttons, self)
end

function game:init()
    -- modes:
    --  normal
    --  travel

    self.mode = "normal"

    self.margin = 75

    self.travelButton = GameButton("Travel", 25, love.graphics.getHeight()-50, nil, 50)
    self.travelButton.activated = function() self.mode = "travel" end
    table.insert(self.uiElements, self.travelButton)

    self.statusButton = GameButton("Status", 25+self:buttonsWidth(), love.graphics.getHeight()-50, nil, 50)
    self.statusButton.activated = function() fx.transition(0.35, status) end
    table.insert(self.uiElements, self.statusButton)

    self.townButton = GameButton("Visit town", 25+self:buttonsWidth(), love.graphics.getHeight()-50, nil, 50)
    self.townButton.activated = function() fx.transition(0.35, town) end
    table.insert(self.uiElements, self.townButton)

    self.exitAreaButton = GameButton("Exit area", 25+self:buttonsWidth(), love.graphics.getHeight()-50, nil, 50)
    self.exitAreaButton.activated = function()
        game.mapLevel = game.mapLevel + 1
        game:genMap()
    end
    table.insert(self.uiElements, self.exitAreaButton)

    self.backgroundImage = love.graphics.newImage('img/bg/sand_papyrus.jpg')


    -- baseline to put frame on
    self.frameLine = 100

    self.frameImage = love.graphics.newImage('img/ui/frame.png')
    self.frameImage:setWrap('repeat', 'repeat')
    self.frameQuad = love.graphics.newQuad(0, 0, love.graphics.getWidth(), self.frameImage:getHeight(), self.frameImage:getWidth(), self.frameImage:getHeight())

    self.goldImage = love.graphics.newImage('img/gold.png')
    self.goldQuad = love.graphics.newQuad(0, 32, 32, 32, self.goldImage:getDimensions())
	
	self.mapSelector = overworldSelector:new(self.map.list, {29, 0, 255})
    self.selector = Selector:new(self.uiElements)
	
	self.mapLevel = 1
	
	--self.itemList = require 'data.items'
	self.itemList = ItemList:new('data/items')
end

function game:enter()
	love.graphics.setBackgroundColor(12, 237, 16)

    assert(self.map.list ~= nil)
    assert(self.map.list[1] ~= nil)

    self.mode = "normal"

    self:updateMenu()
end

function game:buttonsWidth()
    local sum = 0
    for k, button in pairs(self.uiElements) do
        sum = sum + button.width
    end

    return ((#self.uiElements)*self.margin) + sum
end

function game:genMap()
	self.map:empty()
	self.map:generateMap()

	self.player.location = self.map.list[1]
	self.player.x = self.player.location.x - self.player.width/2
	self.player.y = self.player.location.y - self.player.height/2

    -- when you die, if you haven't visited any towns you revive at the spawn point
    self.player.lastTown = self.player.location
	
	self.camera.x = self.player.x + self.player.width/2 - love.graphics.getWidth()/2
	self.camera.y = self.player.y + self.player.height/2 - love.graphics.getHeight()/2
	
	if self.selector then
		self:updateMenu()
	end
end

function game:save(slot, ignoreError)
    self:saveFile(slot, 'player', self.player:toSerialize(), ignoreError)
    self:saveFile(slot, 'map', self.map:toSerialize(), ignoreError)
end

function game:saveFile(slot, component, data, ignoreError)
    if ignoreError ~= false then ignoreError = true end

    local ok, message = pcall(love.filesystem.write, slot..'_'..component, dump(data))

    if not ok and not ignoreError then
        error("could not save storage from disk: " .. message)
    end
end

-- will return true if everything loaded fine, return false otherwise
function game:load(slot, ignoreError)
    -- map
    if not love.filesystem.exists(slot..'_'..'map') then
        return false
    end

    local map, ok = self:loadFile(slot, 'map', ignoreError)
    if ok then
        self.map:fromSerialize(map)
    else
        return ok
    end

    -- player
    if not love.filesystem.exists(slot..'_'..'player') then
        return false
    end

    local player, ok = self:loadFile(slot, 'player', ignoreError)
    if ok then
        self.player:fromSerialize(player)
    else
        return ok
    end

    -- everything went ok
    return true
end

function game:loadFile(slot, component, ignoreError)
    if ignoreError ~= false then ignoreError = true end

    if not love.filesystem.exists(slot..'_'..component) then
        return {}, false
    end

    local ok, data = pcall(love.filesystem.read, slot..'_'..component)

    if ok then
        ok, data = pcall(loadstring('return ' .. data))
        
        if not ok then
            if ignoreError then
                data = {}
            else
                error("could not deserialize storage data: " .. self.data)
            end
        end
    else
        if not ignoreError then
            error("could not load storage from disk: " .. data)
        end
    end

    return data, ok
end

function game:update(dt)
	self.map:update()
	
    if self.mode == "travel" then
	   self.mapSelector:update(dt)
    else
        self.selector:update(dt)
    end
end

function game:keypressed(key, isrepeat)
	if key == "f1" then
		self.mode = self.mode == "normal" and "dev" or "normal"
	end
	
	if key == "f3" then
		game:genMap()
	end
	
	if key == "f5" then
		fx.transition(0.35, shop)
	end

    if key == "escape" and self.mode == "normal" then
        state.push(pause)
    end

    if key == "x" or key == "escape" then
        self.mode = "normal"
    end

    if self.mode == "travel" then
	   self.mapSelector:keypressed(key, isrepeat)
    else
        self.selector:keypressed(key, isrepeat)
    end
end

-- This removes certain buttons when they are not an option
function game:updateMenu()
	self.townVisible = self.player.location.type == "town"
    self.exitVisible = self.player.location.type == "exit"

    local unselectable = {}

    if not self.townVisible then
        table.insert(unselectable, searchTable(self.uiElements, self.townButton))
		-- need to add a way here to deselect the button when it disappears
    end

    if not self.exitVisible then
        table.insert(unselectable, searchTable(self.uiElements, self.exitAreaButton))
    end

    self.selector.unselectable = unselectable
end

function game:enteredLocation(location)
    if location.type == "town" then
        fx.transition(0.35, town)
    elseif math.random() > 0.65 then
        fx.transition(0.35, combat)
    end

    self:updateMenu()
end

function game:draw()
    love.graphics.setColor(255, 255, 255)

    -- background image for top and bottom bars
    local sx = love.graphics.getWidth()/self.backgroundImage:getWidth()
    local sy = love.graphics.getHeight()/self.backgroundImage:getHeight()
    love.graphics.draw(self.backgroundImage, 0, 0, nil, sx, sy)

    -- game background
    love.graphics.setColor(70, 186, 9)
    love.graphics.rectangle("fill", 0, 100, love.graphics.getWidth(), love.graphics.getHeight()-200)

	love.graphics.push()
	love.graphics.translate(-self.camera.x, -self.camera.y)
	--love.graphics.translate(-love.graphics.getWidth()/2, -love.graphics.getHeight()/2)
	--love.graphics.translate(-self.camera.x-love.graphics.getWidth()/2, -self.camera.y-love.graphics.getHeight()/2)
	
	--local scale = 1.5
	--love.graphics.scale(scale, scale)
	--love.graphics.translate(-love.graphics.getWidth()/2/scale, -love.graphics.getHeight()/2/scale)
	
	love.graphics.setScissor(0, self.frameLine, love.graphics.getWidth(), love.graphics.getHeight()-self.frameLine*2)
	self.map:draw()
	self.player:draw()
	love.graphics.pop()
	love.graphics.setScissor()
	
    -- image frames
    love.graphics.setColor(255, 255, 255)
    love.graphics.draw(self.frameImage, self.frameQuad, 0, self.frameLine-self.frameImage:getHeight()/2)
    love.graphics.draw(self.frameImage, self.frameQuad, 0, love.graphics.getHeight()-self.frameLine-self.frameImage:getHeight()/2)

    -- player status
    love.graphics.setFont(font[48])
    love.graphics.print(self.player.name, 25, 25)
	
	if self.townVisible then
		self.townButton:draw()
	end

    if self.exitVisible then
        self.exitAreaButton:draw()
    end
		
    self.travelButton:draw()
    self.statusButton:draw()
	
    if self.mode == "travel" then
	   self.mapSelector:draw()
    else
        self.selector:draw()
    end
	
	if self.mode == "dev" then
		love.graphics.setFont(font[12])
		love.graphics.print("Level: "..self.mapLevel, 3, 20)
	end

    love.graphics.print(self.camera.x .. ', ' .. self.camera.y, 0, 150)

    fx.draw()
end