load = {}

load.uiElements = {}

LoadButton = class("LoadButton", Button)
function LoadButton:initialize(text, slot, x, y, w, h)
    Button.initialize(self, text, x, y, w, h)

    self:setBG(0, 0, 0, 64)
    self:setActive(255, 255, 255)

    self.slot = slot

    local info, ok = game:loadFile(self.slot, 'player')

    self.savePlayerInfo = info
    self.saveExists = ok
    self:edit()

    self.activated = function()
        local ok = game:load(self.slot)
        if ok then
            fx.transition(0.35, game)
        end
    end
end

function LoadButton:reupdate()
    self:edit()

    if not self.saveExists then
        local k = searchTable(load.uiElements, self)
        table.insert(load.selector.unselectable, k)
    end
end

function LoadButton:edit()
    if self.saveExists then
        self.text = self.savePlayerInfo.name
        self:align('left', 25)
    else
        self:setFG(127, 127, 127)
    end
end

function LoadButton:draw()
    Button.draw(self)

    love.graphics.setColor(200, 200, 200)
    love.graphics.setFont(font[16])

    local modtime, errormsg = love.filesystem.getLastModified(tostring(self.slot)..'_player')

    if modtime then
        local text = os.date("%B %d / %Y / %I:%M%p", modtime)
        local font = love.graphics.getFont()

        love.graphics.print(text, self.x+25, self.y+self.height - 32)
    end
end

function load:init()
    self.backgroundImage = love.graphics.newImage('img/bg/papyrus.jpg')

    self.firstSlotButton = LoadButton("Slot 1", 1, nil, nil, 400, 100)
    self.firstSlotButton:centerAround(love.graphics.getWidth()/2, 300)

    self.secondSlotButton = LoadButton("Slot 2", 2, nil, nil, 400, 100)
    self.secondSlotButton:centerAround(love.graphics.getWidth()/2, 450)

    self.thirdSlotButton = LoadButton("Slot 3", 3, nil, nil, 400, 100)
    self.thirdSlotButton:centerAround(love.graphics.getWidth()/2, 600)

    self.backButton = Button("Back")
    self.backButton:setFont(font[36])
    self.backButton:setBG(0, 0, 0, 0)
    self.backButton:setFG(255, 255, 255)
    self.backButton:centerAround(love.graphics.getWidth()/2, 750)
    self.backButton.activated = function()
        fx.transition(0.35, menu)
    end

    table.insert(self.uiElements, self.firstSlotButton)
    table.insert(self.uiElements, self.secondSlotButton)
    table.insert(self.uiElements, self.thirdSlotButton)
    table.insert(self.uiElements, self.backButton)

    self.selector = Selector:new(self.uiElements, 1, "vertical")
end

function load:enter()
    self.firstSlotButton:reupdate()
    self.secondSlotButton:reupdate()
    self.thirdSlotButton:reupdate()
end

function load:keypressed(key, isrepeat)
    self.selector:keypressed(key, isrepeat)

    if key == "escape" then
        fx.transition(0.35, menu)
    end
end

function load:draw()
    -- background image for top and bottom bars
    local sx = love.graphics.getWidth()/self.backgroundImage:getWidth()
    local sy = love.graphics.getHeight()/self.backgroundImage:getHeight()
    love.graphics.draw(self.backgroundImage, 0, 0, nil, sx, sy)

    love.graphics.setFont(font[36])
    love.graphics.setColor(255, 255, 255)
    local w = love.graphics.getFont():getWidth("Load game")
    love.graphics.print("Load game", love.graphics.getWidth()/2-w/2, 125)

    self.firstSlotButton:draw()
    self.secondSlotButton:draw()
    self.thirdSlotButton:draw()
    self.backButton:draw()

    self.selector:draw()

    fx.draw()
end