menu = {}

menu.sound = {}
menu.buttons = {}

-- MenuButton differs from button just a little bit
-- mostly for style/width adjustments
MenuButton = class("MenuButton", Button)
function MenuButton:initialize(text, x, y, w, h)
    Button.initialize(self, text, x, y, w, h)

    self:setFont(font[32])
    self:align("y")
    self:setBG(22, 22, 22, 0)
    self:setFG(255, 255, 255)

    table.insert(menu.buttons, self)
end

function menu:init()
    love.graphics.setBackgroundColor(22, 22, 22)

    self.sound.theme = love.audio.newSource('sound/soliloquy.mp3', 'stream')
    self.sound.themeVolume = 1.0

    playSound(self.sound.theme, self.sound.themeVolume)

    self.banner = love.graphics.newImage('img/bg/vista.jpg')
    self.backgroundImage = love.graphics.newImage('img/bg/papyrus.jpg')

    -- baseline to place buttons on
    self.buttonLine = love.graphics.getHeight() - (love.graphics.getHeight() - self.banner:getHeight())/4

    -- baseline to put frame on
    self.frameLine = (love.graphics.getHeight() - self.banner:getHeight())/2

    self.margin = 75

    self.startButton = MenuButton("New game", 0, self.buttonLine)
    self.loadButton = MenuButton("Load", self.startButton.width+self.margin, self.buttonLine)
    self.quitButton = MenuButton("Exit", self.startButton.width+self.loadButton.width+self.margin*2, self.buttonLine)

    self.startButton.activated = function() fx.transition(0.35, start) end
    self.loadButton.activated = function() fx.transition(0.35, load) end
    self.quitButton.activated = love.event.quit

    -- calculate buttons width
    self.buttonsWidth = 0
    for k, button in pairs(self.buttons) do
        self.buttonsWidth = self.buttonsWidth + button.width
    end
    self.buttonsWidth = self.buttonsWidth + self.margin*(#self.buttons-1)

    -- center buttons around center of the screen
    for k, button in pairs(self.buttons) do
        button.x = button.x + love.graphics.getWidth()/2 - self.buttonsWidth/2
    end

    self.frameImage = love.graphics.newImage('img/ui/frame.png')
    self.frameImage:setWrap('repeat', 'repeat')
    self.frameQuad = love.graphics.newQuad(0, 0, love.graphics.getWidth(), self.frameImage:getHeight(), self.frameImage:getWidth(), self.frameImage:getHeight())
	
	self.selector = Selector:new(self.buttons, 1)
	self.startButton.selected = true
	
	
	self.mode = 1
	self.modeMax = 2
end

function menu:update(dt)
	self.selector:update(dt)
	
	if self.mode == 2 then
		local slope = love.mouse.getX()/100
		local w = love.mouse.getY()/1000
		--self.shader:send("slope", slope)
		--self.shader:send("w", w)
		self.shader:send("mouseX", love.mouse:getX())
	end
end

function menu:draw()
    -- background image
    local sx = love.graphics.getWidth()/self.backgroundImage:getWidth()
    local sy = love.graphics.getHeight()/self.backgroundImage:getHeight()
    love.graphics.draw(self.backgroundImage, 0, 0, nil, sx, sy)

    -- banner image
    love.graphics.setColor(255, 255, 255)
    local x = 0
    local y = (love.graphics.getHeight() - self.banner:getHeight())/2
	
	if self.mode > 1 then love.graphics.setShader(self.shader) end
    love.graphics.draw(self.banner, x, y)
	love.graphics.setShader()

    -- title text
    local text = config.gameTitle
    love.graphics.setFont(font[48])

    -- outline
    love.graphics.setColor(0, 0, 0)
    love.graphics.print(text, 52, 42)

    -- main text
    love.graphics.setColor(255, 255, 255)
    love.graphics.print(text, 50, 40)

    -- image frames
    love.graphics.draw(self.frameImage, self.frameQuad, 0, self.frameLine-self.frameImage:getHeight()/2)
    love.graphics.draw(self.frameImage, self.frameQuad, 0, love.graphics.getHeight()-self.frameLine-self.frameImage:getHeight()/2)

    -- buttons
    self.startButton:draw()
    self.loadButton:draw()
    self.quitButton:draw()
	
	self.selector:draw()

    fx.draw()
end

function menu:leave()
    self.sound.theme:stop()
end

function menu:keypressed(key, isrepeat)
    self.selector:keypressed(key, isrepeat)
	
	if key == 'rshift' or key == 'lshift' then
		if self.mode < self.modeMax then
			self.mode = self.mode + 1
		else
			self.mode = 1
		end
		
		if self.mode == 2 then
			self.shader = love.graphics.newShader('shaders/custom.glsl')
		end
	end
end

function menu:keyreleased(key, code)

end