save = {}

save.uiElements = {}

SaveButton = class("LoadButton", Button)
function SaveButton:initialize(text, slot, x, y, w, h)
    Button.initialize(self, text, x, y, w, h)

    self:setBG(0, 0, 0, 64)
    self:setActive(255, 255, 255)

    self.slot = slot

    local info, ok = game:loadFile(self.slot, 'player')

    self.savePlayerInfo = info
    self.saveExists = ok
    self:edit()

    self.activated = function()
        game:save(self.slot)
        if pause.savingAndQuitting then
            fx.transition(0.35, menu)
        end
    end
end

function SaveButton:reupdate()
    self:edit()
end

function SaveButton:edit()
    if self.saveExists then
        self.text = self.savePlayerInfo.name
        self:align('left', 25)
    else
        self:setFG(127, 127, 127)
    end
end

function SaveButton:draw()
    Button.draw(self)

    love.graphics.setColor(200, 200, 200)
    love.graphics.setFont(font[16])

    local modtime, errormsg = love.filesystem.getLastModified(tostring(self.slot)..'_player')

    if modtime then
        local text = os.date("%B %d / %Y / %I:%M%p", modtime)
        local font = love.graphics.getFont()

        love.graphics.print(text, self.x+25, self.y+self.height - 32)
    end
end

function save:init()
    self.backgroundImage = love.graphics.newImage('img/bg/papyrus.jpg')

    self.firstSlotButton = SaveButton("Slot 1", 1,  nil, nil, 400, 100)
    self.firstSlotButton:centerAround(love.graphics.getWidth()/2, 300)
    table.insert(self.uiElements, self.firstSlotButton)

    self.secondSlotButton = SaveButton("Slot 2", 2, nil, nil, 400, 100)
    self.secondSlotButton:centerAround(love.graphics.getWidth()/2, 450)
    table.insert(self.uiElements, self.secondSlotButton)

    self.thirdSlotButton = SaveButton("Slot 3", 3, nil, nil, 400, 100)
    self.thirdSlotButton:centerAround(love.graphics.getWidth()/2, 600)
    table.insert(self.uiElements, self.thirdSlotButton)

    self.backButton = Button("Back")
    self.backButton:setFont(font[36])
    self.backButton:setBG(0, 0, 0, 0)
    self.backButton:setFG(255, 255, 255)
    self.backButton:centerAround(love.graphics.getWidth()/2, 750)
    self.backButton.activated = function() fx.transition(0.35, game) end
    table.insert(self.uiElements, self.backButton)

    self.selector = Selector:new(self.uiElements, 1, "vertical")
end

function save:enter()
    self.firstSlotButton:reupdate()
    self.secondSlotButton:reupdate()
    self.thirdSlotButton:reupdate()
end

function save:update()
    self.selector:update(dt)
end

function save:keypressed(key, isrepeat)
    self.selector:keypressed(key, isrepeat)
end

function save:draw()

    -- background image for top and bottom bars
    local sx = love.graphics.getWidth()/self.backgroundImage:getWidth()
    local sy = love.graphics.getHeight()/self.backgroundImage:getHeight()
    love.graphics.draw(self.backgroundImage, 0, 0, nil, sx, sy)

    love.graphics.setFont(font[36])
    love.graphics.setColor(255, 255, 255)
    local w = love.graphics.getFont():getWidth("Save game")
    love.graphics.print("Save game", love.graphics.getWidth()/2-w/2, 125)

    self.firstSlotButton:draw()
    self.secondSlotButton:draw()
    self.thirdSlotButton:draw()

    self.backButton:draw()

    self.selector:draw()

    fx.draw()
end