shop = {}

shop.sound = {}
shop.buttons = {}

ShopButton = class("ShopButton", Button)

function ShopButton:initialize(text, x, y, w, h)
    Button.initialize(self, text, x, y, w, h)

    self:setFont(font[32])
    self:align("y")
    self:setBG(22, 22, 22, 0)
    self:setFG(255, 255, 255)
    table.insert(shop.buttons, self)
end

function shop:init()
	self.backgroundImage = love.graphics.newImage('img/bg/sand_papyrus.jpg')

	self.items = {}
	for k, item in ipairs(game.itemList.items) do
		if string.lower(item.shop) == 'true' or string.lower(item.shop) == 'yes' then
			table.insert(self.items, item)
		end
	end

	
	for i = 1, #self.items do
		self.items[i].xModifier = 0
	end
	
	self.itemTopMargin = 20
	self.itemLeftMargin = 30
	self.itemRightMargin = 30
	self.itemHeight = 50
	self.itemWidth = 200
	self.itemSpacing = 0
	
	-- baseline to put frame on
    self.frameLine = 100
	
	self.vertFrameLine = 500
	
	self.descTopMargin = 30
	self.descLeftMargin = 20

    self.frameImage = love.graphics.newImage('img/ui/frame.png')
    self.frameImage:setWrap('repeat', 'repeat')
    self.frameQuad = love.graphics.newQuad(0, 0, love.graphics.getWidth(), self.frameImage:getHeight(), self.frameImage:getWidth(), self.frameImage:getHeight())

    self.frameVertImage = love.graphics.newImage('img/ui/frame_vert.png')
    self.frameVertImage:setWrap('repeat', 'repeat')
    self.frameVertQuad = love.graphics.newQuad(0, 0, self.frameVertImage:getWidth(), love.graphics.getHeight() - 2*self.frameLine - self.frameImage:getHeight() - self.frameImage:getHeight()/2, self.frameVertImage:getWidth(), self.frameVertImage:getHeight())

	
	
	self.active = 1
	--self.selector = Selector:new(self.items)
	
	self.bannerImg = love.graphics.newImage('img/ui/shopBanner.png')
	self.itemImg = love.graphics.newImage('img/ui/shopItem.png')
	self.itemDesc = love.graphics.newImage('img/ui/shopDesc.png')
end

function shop:mousepressed(x, y, button)
    --if self.quitButton:clicked() then
    --    state.switch(game)
    --end
	for k, button in ipairs(self.buttons) do
		if button:clicked() then
			shop:nextLine(k)
			break
		end
	end
end

function shop:keypressed(key, isrepeat)
	if key == "up" then
		--tween(0.4, self.items[self.active], {xModifier = 0})
		self.active = self.active > 1 and self.active - 1 or #self.items
		--tween(0.4, self.items[self.active], {xModifier = 20})
	elseif key == "down" then
		--tween(0.4, self.items[self.active], {xModifier = 0})
		self.active = self.active < #self.items and self.active + 1 or 1
		--tween(0.4, self.items[self.active], {xModifier = 20})
	elseif key == " " then
		shop:purchase()
	elseif key == "escape" then
		fx.transition(0.35, game)
	end
	--self.selector:keypressed(key, isrepeat)
end

function shop:purchase()
	local item = self.items[self.active]
	if 30 < game.player.gold then
		--game.player.gold = game.player.gold - item.price
		game.player:modifyInv("gold", "take", item.price)
		
		if item.itemType == "weapon" then
			game.player:modifyInv("weapon", "give", item.name, item.minDamage, item.maxDamage)
		end
	end
end

function shop:update(dt)
	--self.selector:update(dt)
end

function shop:draw()
    love.graphics.setColor(255, 255, 255)
    love.graphics.setFont(font[48])

    -- background image for top and bottom bars
    local sx = love.graphics.getWidth()/self.backgroundImage:getWidth()
    local sy = love.graphics.getHeight()/self.backgroundImage:getHeight()
    love.graphics.draw(self.backgroundImage, 0, 0, nil, sx, sy)
    
    -- image frames
    love.graphics.setColor(255, 255, 255)
    love.graphics.draw(self.frameImage, self.frameQuad, 0, self.frameLine)
    love.graphics.draw(self.frameImage, self.frameQuad, 0, love.graphics.getHeight()-self.frameLine-self.frameImage:getHeight()/2)

    love.graphics.draw(self.frameVertImage, self.frameVertQuad, self.vertFrameLine, self.frameLine + self.frameImage:getHeight())

	
	
	
	for k, v in ipairs(self.items) do
		local x = self.itemLeftMargin - v.xModifier
		local y = 100 + self.itemTopMargin + (self.itemHeight + self.itemSpacing) * k - self.itemTopMargin
		love.graphics.setColor(255, 255, 255)
		--love.graphics.rectangle("fill", x, y, self.itemWidth, self.itemHeight)
		--love.graphics.draw(self.itemImg, x-5, y+4)
		love.graphics.setColor(0, 0, 0)
		if k == self.active then
			love.graphics.setColor(2, 221, 232)
		end
		love.graphics.print(v.name.." "..v.price, x, y)
	end
	
	love.graphics.setColor(255, 255, 255)
	
	
	--love.graphics.draw(self.itemDesc, self.itemLeftMargin + 370, 90)
	
	local activeItem = self.items[self.active]
	local str = activeItem.name .. "\n" .. activeItem.price .. ' gold'
	if activeItem.damageMin and activeItem.damageMax then str = str .. '\n\nDamage: ' .. activeItem.damageMin .. ' - ' .. activeItem.damageMax end
	
	str = str .. '\n' .. activeItem.description
	
	local limit = love.graphics:getWidth() - self.vertFrameLine - self.itemLeftMargin - self.itemRightMargin
	love.graphics.printf(str, self.vertFrameLine + self.descLeftMargin, self.frameLine + self.descTopMargin, limit)
	
	
	--love.graphics.draw(self.bannerImg, 0, 0)
	shop:drawGold()
	
	--self.selector:draw()
end

function shop:drawGold()
    love.graphics.push()

    local text = tostring(game.player.gold)
    love.graphics.setFont(font[32])

    local scaleFactor = 2
    local goldImageWidth = 32*scaleFactor
    local margin = 8
    local textWidth = love.graphics.getFont():getWidth(text)
    local sum = goldImageWidth + margin + textWidth

    love.graphics.translate(love.graphics.getWidth() - sum - 64, 0)

    love.graphics.print(text, sum - textWidth, 25)

    local x = 0
    local y = 25 - 16
    love.graphics.draw(game.goldImage, game.goldQuad, x, y, nil, 2, 2)

    love.graphics.pop()
end