status = {}

status.mode = "normal"
status.buttons = {}
status.uiElements = {}

StatusButton = class("MenuButton", Button)
function StatusButton:initialize(text, x, y, w, h)
    Button.initialize(self, text, x, y, w, h)

    self:setFont(font[32])
    self:align("y")
    self:setBG(22, 22, 22, 0)
    self:setFG(255, 255, 255)

    table.insert(status.buttons, self)
end

function status:init()
    self.backButton = StatusButton("Back", 25, love.graphics.getHeight()-50, nil, 50)
    self.backButton.activated = function() fx.transition(0.35, game) end 
    table.insert(self.uiElements, self.backButton)

    self.backgroundImage = love.graphics.newImage('img/bg/sand_papyrus.jpg')
    self.heartImage = love.graphics.newImage('img/item/128/heart.png')
    self.swordImage = love.graphics.newImage('img/item/128/sword.png')
    self.shieldImage = love.graphics.newImage('img/item/128/shield.png')
    self.coinImage = love.graphics.newImage('img/item/128/coin.png')

    -- baseline to put frame on
    self.frameLine = 100

    self.frameImage = love.graphics.newImage('img/ui/frame.png')
    self.frameImage:setWrap('repeat', 'repeat')
    self.frameQuad = love.graphics.newQuad(0, 0, love.graphics.getWidth(), self.frameImage:getHeight(), self.frameImage:getWidth(), self.frameImage:getHeight())

    self.frameVertImage = love.graphics.newImage('img/ui/frame_vert.png')
    self.frameVertImage:setWrap('repeat', 'repeat')
    self.frameVertQuad = love.graphics.newQuad(0, 0, self.frameVertImage:getWidth(), love.graphics.getHeight() - 2*self.frameLine - self.frameImage:getHeight() - self.frameImage:getHeight()/2, self.frameVertImage:getWidth(), self.frameVertImage:getHeight())

    self.goldImage = love.graphics.newImage('img/gold.png')
    self.goldQuad = love.graphics.newQuad(0, 32, 32, 32, self.goldImage:getDimensions())
    
    self.selector = Selector:new(self.uiElements)
end

function status:enter()
    love.graphics.setBackgroundColor(33, 33, 33)
end

function status:update(dt)
    self.selector:update(dt)
end

function status:keypressed(key, isrepeat)
    if key == "x" or key == "escape" then
        fx.transition(0.35, game)
    end

    self.selector:keypressed(key, isrepeat)
end

function status:draw()
    love.graphics.setColor(255, 255, 255)

    -- background image for top and bottom bars
    local sx = love.graphics.getWidth()/self.backgroundImage:getWidth()
    local sy = love.graphics.getHeight()/self.backgroundImage:getHeight()
    love.graphics.draw(self.backgroundImage, 0, 0, nil, sx, sy)
    
    -- image frames
    love.graphics.setColor(255, 255, 255)
    love.graphics.draw(self.frameImage, self.frameQuad, 0, self.frameLine)
    love.graphics.draw(self.frameImage, self.frameQuad, 0, love.graphics.getHeight()-self.frameLine-self.frameImage:getHeight()/2)

    love.graphics.draw(self.frameVertImage, self.frameVertQuad, 400, self.frameLine + self.frameImage:getHeight())

    -- player status
    love.graphics.setFont(font[48])
    love.graphics.print(game.player.name, 25, 25)

    love.graphics.push()

    local player = game.player
    love.graphics.draw(self.heartImage, 0, 150)
    love.graphics.setFont(font[32])
    love.graphics.print(tostring(player.health)..' / '..tostring(player.maxHealth).. ' health', 128, 190)

    love.graphics.translate(0, 150)
    love.graphics.setColor(255, 255, 255)

    -- weapon stats
    local weapon = game.player.weapon
    love.graphics.draw(self.swordImage, 0, 150)
    love.graphics.setFont(font[32])
    love.graphics.print(weapon.name, 128, 160)
    love.graphics.setFont(font[18])
    love.graphics.setColor(200, 200, 200)
    love.graphics.print(tostring(weapon.minDamage)..' to '..tostring(weapon.maxDamage)..' attack damage', 128, 195)

    local critPercentage = (weapon.critRange[2] - weapon.critRange[1] + 1) * 5
    love.graphics.print(tostring(critPercentage)..'% critical hit chance', 128, 215)

    love.graphics.translate(0, 150)
    love.graphics.setColor(255, 255, 255)

    -- armor stats
    local armor = game.player.armor
    love.graphics.draw(self.shieldImage, 0, 150)
    love.graphics.setFont(font[32])
    love.graphics.print(armor.name, 128, 160)
    love.graphics.setFont(font[22])
    love.graphics.setColor(200, 200, 200)
    love.graphics.print(tostring(armor.defense)..' defense', 128, 200)

    love.graphics.translate(0, 150)
    love.graphics.setColor(255, 255, 255)

    -- armor stats
    love.graphics.draw(self.coinImage, 0, 150)
    love.graphics.setFont(font[32])
    love.graphics.print(tostring(player.gold)..' gold', 128, 190)

    love.graphics.pop()

    self.backButton:draw()

    self.selector:draw()

    fx.draw()
end