town = {}

town.sound = {}
town.uiElements = {}

TownButton = class("TownButton", Button)
function TownButton:initialize(text, x, y, w, h)
    Button.initialize(self, text, x, y, w, h)

    self:setFont(font[32])
    self:align("y")
    self:setBG(22, 22, 22, 0)
    self:setFG(255, 255, 255)
    table.insert(town.uiElements, self)
end

function town:init()
	self.sound.theme = love.audio.newSource('sound/townTheme.mp3', 'stream')
    self.sound.themeVolume = .5

    love.graphics.setBackgroundColor(33, 33, 33)

    playSound(self.sound.theme, self.sound.themeVolume)

    self.banner = love.graphics.newImage('img/bg/town.jpg')
    self.backgroundImage = love.graphics.newImage('img/bg/papyrus.jpg')

    -- baseline to place buttons on
    self.buttonLine = love.graphics.getHeight() - (love.graphics.getHeight() - self.banner:getHeight())/4

    -- baseline to put frame on
    self.frameLine = (love.graphics.getHeight() - self.banner:getHeight())/2

    self.margin = 75

    self.buttonsWidth = function()
        local sum = 0
        for k, button in pairs(self.uiElements) do
            sum = sum + button.width
        end

        return ((#self.uiElements)*self.margin) + sum
    end

    self.innButton = TownButton("Tavern", 0, self.buttonLine)
    self.innButton.activated = function() fx.transition(0.35, tavern) end

    self.shopButton = TownButton("Shoppe", self.buttonsWidth(), self.buttonLine)
    self.shopButton.activated = function() fx.transition(0.35, shop) end

    self.squareButton = TownButton("Town Square", self.buttonsWidth(), self.buttonLine)
    self.squareButton.activated = function() end

    self.quitButton = TownButton("Leave", self.buttonsWidth(), self.buttonLine)
    self.quitButton.activated = function() fx.transition(0.35, game) end

    -- center buttons around center of the screen
    for k, button in pairs(self.uiElements) do
        button.x = button.x + love.graphics.getWidth()/2 - self.buttonsWidth()/2 + self.margin/2
    end

    self.frameImage = love.graphics.newImage('img/ui/frame.png')
    self.frameImage:setWrap('repeat', 'repeat')
    self.frameQuad = love.graphics.newQuad(0, 0, love.graphics.getWidth(), self.frameImage:getHeight(), self.frameImage:getWidth(), self.frameImage:getHeight())
	
	--texter:add("welcome to our village!", 22, 22, {0, 0, 0}, 50, "left")
	
	self.selector = Selector:new(self.uiElements)
end

function town:leave()
    self.sound.theme:stop()
end

function town:update(dt)
    self.selector:update(dt)
end

function town:keypressed(key, isrepeat)
    self.selector:keypressed(key, isrepeat)
end

function town:draw()

	-- background image
    local sx = love.graphics.getWidth()/self.backgroundImage:getWidth()
    local sy = love.graphics.getHeight()/self.backgroundImage:getHeight()
    love.graphics.draw(self.backgroundImage, 0, 0, nil, sx, sy)

    -- banner image
    love.graphics.setColor(255, 255, 255)
    local x = 0
    local y = (love.graphics.getHeight() - self.banner:getHeight())/2
    love.graphics.draw(self.banner, x, y)

    -- title text
    local text = "Hello, welcome to NuthIkroth-ville!"
    love.graphics.setFont(font[48])

    -- outline
    love.graphics.setColor(0, 0, 0)
    love.graphics.print(text, 52, 42)

    -- main text
    love.graphics.setColor(255, 255, 255)
    love.graphics.print(text, 50, 40)

    -- image frames
    love.graphics.draw(self.frameImage, self.frameQuad, 0, self.frameLine-self.frameImage:getHeight()/2)
    love.graphics.draw(self.frameImage, self.frameQuad, 0, love.graphics.getHeight()-self.frameLine-self.frameImage:getHeight()/2)

    -- buttons
    self.innButton:draw()
    self.shopButton:draw()
    self.squareButton:draw()
    self.quitButton:draw()

    self.selector:draw()

	
	--text:drawStuff()
	texter:draw()
    fx.draw()
end